/************************************************************************
 *   IRC - Internet Relay Chat, m_core/m_kick.c
 *   This file is copyright (C) 2001 Andrew Suffield
 *                                    <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "ircd.h"
#include "common.h"
#include "numeric.h"
#include "irc_string.h"
#include "send.h"
#include "umodes.h"
#include "msg.h"
#include "handlers.h"
#include "channel.h"
#include "hash.h"

#include <string.h>

int m_remove(struct Client *, struct Client *, int, char **);

struct Message remove_msgtab = {
  "REMOVE", 0, 3, 0, MFLG_SLOW, 0,
  {m_unregistered, m_remove, m_remove},
  NULL, NULL
};

static char buf[BUFSIZE], reasonbuf[BUFSIZE];

/* OK, this is just like KICK, but it sends out PARTs. For testing of this
 * idea, I want to see how well clients handle it
 *  -- asuffield
 */

int
m_remove(struct Client *cptr, struct Client *sptr, int parc, char *parv[])
{
  struct Client *who;
  struct Channel *chptr;
  int chasing = 0;
  char *comment;
  char *name;
  char *p = (char *) NULL;
  char *user;

  if (!HasUmode(sptr, UMODE_EXPERIMENTAL))
    {
      if (SeesOperMessages(sptr))
	sendto_one(sptr, get_str(STR_NEEDPRIV), me.name, parv[0],
		   user_mode_char[UMODE_EXPERIMENTAL]);
      else
	sendto_one(sptr, form_str(ERR_NOPRIVILEGES), me.name, parv[0]);
      return 0;
    }

  if (parc < 3 || *parv[1] == '\0')
    {
      sendto_one(sptr, form_str(ERR_NEEDMOREPARAMS),
		 me.name, parv[0], "REMOVE");
      return 0;
    }
  if (IsServer(sptr))
    sendto_local_ops_flag(UMODE_SPY, get_str(STR_KICK_SPY),	/* "KICK by %s on %s, target %s" */
			  parv[0], parv[1], parv[2]);
  comment = reasonbuf;
  if (BadPtr(parv[3]))
    {
      ircsprintf(comment, "Removed by %s", parv[0]);
    }
  else
    {
      ircsprintf(comment, "Removed by %s: \"%s\"", parv[0], parv[3]);
    }
  if (strlen(comment) > (size_t) TOPICLEN)
    comment[TOPICLEN] = '\0';

  *buf = '\0';
  p = strchr(parv[1], ',');
  if (p)
    *p = '\0';
  name = parv[1];		/* strtoken(&p, parv[1], ","); */

  chptr = get_channel(sptr, name, !CREATE);
  if (!chptr)
    {
      sendto_one(sptr, form_str(ERR_NOSUCHCHANNEL), me.name, parv[0], name);
      return (0);
    }

  /* You either have chan op privs, or you don't -Dianora */
  /* orabidoo and I discussed this one for a while...
   * I hope he approves of this code, (he did) users can get quite confused...
   *    -Dianora
   */

  if (!IsServer(sptr) && !is_chan_op(sptr, chptr)
      && !HasUmode(sptr, UMODE_GOD))
    {
      /* was a user, not a server, and user isn't seen as a chanop here */

      if (MyConnect(sptr))
	{
	  /* user on _my_ server, with no chanops.. so go away */

	  sendto_one(sptr, form_str(ERR_CHANOPRIVSNEEDED),
		     me.name, parv[0], chptr->chname);
	  return (0);
	}

      if (chptr->channelts == 0)
	{
	  /* If its a TS 0 channel, do it the old way */

	  sendto_one(sptr, form_str(ERR_CHANOPRIVSNEEDED),
		     me.name, parv[0], chptr->chname);
	  return (0);
	}

      /* Its a user doing a kick, but is not showing as chanop locally
       * its also not a user ON -my- server, and the channel has a TS.
       * There are two cases we can get to this point then...
       *
       *     1) connect burst is happening, and for some reason a legit
       *        op has sent a KICK, but the SJOIN hasn't happened yet or 
       *        been seen. (who knows.. due to lag...)
       *
       *     2) The channel is desynced. That can STILL happen with TS
       *        
       *     Now, the old code roger wrote, would allow the KICK to 
       *     go through. Thats quite legit, but lets weird things like
       *     KICKS by users who appear not to be chanopped happen,
       *     or even neater, they appear not to be on the channel.
       *     This fits every definition of a desync, doesn't it? ;-)
       *     So I will allow the KICK, otherwise, things are MUCH worse.
       *     But I will warn it as a possible desync.
       *
       *     -Dianora
       */

      /*          sendto_one(sptr, form_str(ERR_DESYNC),
       *           me.name, parv[0], chptr->chname);
       */

      /*
       * After more discussion with orabidoo...
       *
       * The code was sound, however, what happens if we have +h (TS4)
       * and some servers don't understand it yet? 
       * we will be seeing servers with users who appear to have
       * no chanops at all, merrily kicking users....
       * -Dianora
       */
    }

  p = strchr(parv[2], ',');
  if (p)
    *p = '\0';
  user = parv[2];		/* strtoken(&p2, parv[2], ","); */

  if (!(who = find_chasing(sptr, user, &chasing)))
    {
      return (0);
    }

  if (IsMember(who, chptr))
    {
      /* Immune users */
      if (!(IsServer(sptr) || HasUmode(sptr, UMODE_GOD))
	  && HasUmode(who, UMODE_IMMUNE))
	{
	  sendto_one(sptr, form_str(ERR_USERISIMMUNE),
		     me.name, parv[0], user, name);
	  return (0);
	}

#ifdef HIDE_OPS
      sendto_channel_chanops_butserv(chptr, sptr,
				     ":%s PART %s :%s", who->name,
				     name, comment);
      sendto_channel_non_chanops_butserv(chptr, sptr,
					 ":%s PART %s :%s", who->name,
					 name, comment);

#else
      sendto_channel_butserv(chptr, sptr,
			     ":%s PART %s :%s", who->name, name, comment);
#endif
      sendto_match_servs(chptr, cptr,
			 ":%s PART %s :%s", who->name, name, comment);
      remove_user_from_channel(who, chptr, 1);
      if (MyClient(sptr))
	sendto_ops_flag(UMODE_BOTS,
			"Remove from %s by %s on %s (%s@%s) [%s]",
			name, sptr->name, who->name,
			who->username, who->host, who->user->server);
    }
  else
    sendto_one(sptr, form_str(ERR_USERNOTINCHANNEL),
	       me.name, parv[0], user, name);

  return (0);
}
